#!/bin/python

import time
import uuid
import os
import sys
import signal
import traceback
import argparse
import logging

from zmsclient.zmc.client import ZmsZmcClient
from zmsclient.dst.client import ZmsDstClient
from zmsclient.dst.v1.models import Subscription, Observation
from zmsclient.common.subscription import ZmsSubscriptionCallback
from . import DefaultDestEnvAction

LOG = logging.getLogger(__name__)

# Not sure where to pick these up
EVENT_TYPE_CREATED     =     2
EVENT_SOURCETYPE_DST   =     3
EVENT_CODE_OBSERVATION =     3001

# Only when a daemon
LOGFILE = "/local/logs/zms-observations-relay.log"

#
# Need a map of outer monitor IDs to inner monitor IDs.
#
def loadMonitorMap(inner_zmc_client, outer_zmc_client, monitor=None):
    monitorMap = {}
    kwargs = dict()
    if monitor:
        kwargs["monitor"] = monitor
    inner_monitors = inner_zmc_client.list_monitors(**kwargs, items_per_page=1000)
    outer_monitors = outer_zmc_client.list_monitors(**kwargs, items_per_page=1000)

    for inner_monitor in inner_monitors.monitors:
        if not inner_monitor.enabled:
            continue

        for outer_monitor in outer_monitors.monitors:
            if outer_monitor.id == inner_monitor.device_id:
                monitorMap[outer_monitor.id] = inner_monitor.id;
                break

    LOG.info("Monitor map: %r", monitorMap)
    return monitorMap

#
# Inject an observation from the parent, into the child.
#
def injectObservation(outer_dst_client, inner_dst_client, id, element_id,
                      monitor_map=dict(), impotent=False):
    observation = outer_dst_client.get_observation(id, data_inline=True)
    inner_monitor_id = monitor_map.get(observation.monitor_id, None)
    LOG.info("Observation: %r", id)
    LOG.info("  Outer Monitor: %r", observation.monitor_id)
    LOG.info("  Inner Monitor: %r", inner_monitor_id)

    new = Observation(element_id=element_id,
                      monitor_id=inner_monitor_id,
                      types=observation.types,
                      format_=observation.format_,
                      description=observation.description,
                      min_freq=observation.min_freq,
                      max_freq=observation.max_freq,
                      starts_at=observation.starts_at,
                      violation=observation.violation)

    LOG.debug("Creating observation: %r", new)
    if impotent:
        return

    new.data = observation.data
    response = inner_dst_client.create_observation(body=new)
    LOG.debug("created observation: %r", response)

#
# Subclass ZmsSubscriptionCallback, to inject observations from the outer ZMS,
# into the inner ZMS
#
class MyZmsSubscriptionCallback(ZmsSubscriptionCallback):
    def __init__(self, outer_dstclient, inner_dstclient, inner_element_id,
                 monitor_map=dict(), impotent=False, **kwargs):
        super(MyZmsSubscriptionCallback, self).__init__(outer_dstclient, **kwargs)
        self.outerDST = outer_dstclient
        self.innerDST = inner_dstclient
        self.inner_element_id = inner_element_id
        self.monitor_map = monitor_map
        self.impotent = impotent

    def on_event(self, ws, evt, message):
        # For now, just observations
        if evt.header.source_type != EVENT_SOURCETYPE_DST:
            LOG.error("on_event: unexpected source type: %r (%r)",
                      evt.header.source_type, message)
            return
        if evt.header.type != EVENT_TYPE_CREATED:
            LOG.error("on_event: unexpected type: %r (%r)",
                      evt.header.type, message)
            return
        if evt.header.code != EVENT_CODE_OBSERVATION:
            LOG.error("on_event: unexpected code: %r (%r)",
                      evt.header.code, message)
            return

        try:
            injectObservation(
                self.outerDST, self.innerDST, evt.header.object_id,
                self.inner_element_id, monitor_map=self.monitor_map,
                impotent=self.impotent)
        except Exception as ex:
            LOG.exception(ex)

def main():
    parser = argparse.ArgumentParser(
        prog="relayevents",
        description="Relay observation events from a 'parent' OpenZMS to another.")
    parser.add_argument(
        "-b", "--daemon", default=False, action="store_true",
        help="Daemonize")
    parser.add_argument(
        "-d", "--debug", default=0, action="count",
        help="Increase debug level: defaults to INFO; add once for zmsclient DEBUG; add twice to set the root logger level to DEBUG")
    parser.add_argument(
        "-n", "--impotent", default=False, action="store_true",
        help="Impotent: do not inject events")
    parser.add_argument(
        "--logfile", default=LOGFILE, type=str,
        help="Redirect logging to a file when daemonizing.")
    parser.add_argument(
        "--element-id", action=DefaultDestEnvAction, type=str, required=True,
        help="Local (inner) element ID")
    parser.add_argument(
        "--element-token", action=DefaultDestEnvAction, type=str, required=True,
        help="Local (inner) element token")
    parser.add_argument(
        "--zmc-http", action=DefaultDestEnvAction, type=str, required=True,
        help="Local (inner) ZMC URL")
    parser.add_argument(
        "--dst-http", action=DefaultDestEnvAction, type=str, required=True,
        help="Local (inner) DST URL")
    parser.add_argument(
        "--parent-zmc-http", action=DefaultDestEnvAction, type=str, required=True,
        help="Upstream (outer) ZMC URL")
    parser.add_argument(
        "--parent-dst-http", action=DefaultDestEnvAction, type=str, required=True,
        help="Upstream (outer) DST URL")
    parser.add_argument(
        "--parent-element-token", action=DefaultDestEnvAction, type=str, required=True,
        help="Upstream (outer) element token")
    parser.add_argument(
        "--monitor", default=None, type=str,
        help="Filter by monitor name")

    args = parser.parse_args(sys.argv[1:])

    if args.debug:
        LOG.setLevel(logging.DEBUG)
        logging.getLogger('zmsclient').setLevel(logging.DEBUG)
    if args.debug > 1:
        logging.getLogger().setLevel(logging.DEBUG)

    innerZMC   = ZmsZmcClient(args.zmc_http, args.element_token,
                              detailed=False, raise_on_unexpected_status=True)
    innerDST   = ZmsDstClient(args.dst_http, args.element_token,
                              detailed=False, raise_on_unexpected_status=True)
    outerZMC   = ZmsZmcClient(args.parent_zmc_http, args.parent_element_token,
                              detailed=False, raise_on_unexpected_status=True)
    outerDST   = ZmsDstClient(args.parent_dst_http, args.parent_element_token,
                              detailed=False, raise_on_unexpected_status=True)

    # Map outer monitors to inner monitors
    monitor_map = loadMonitorMap(innerZMC, outerZMC, monitor=args.monitor)

    # And create the subscription (which includes the websocket)
    subscription = MyZmsSubscriptionCallback(
        outerDST, innerDST, args.element_id,
        subscription=Subscription(id=str(uuid.uuid4())),
        monitor_map=monitor_map,
        impotent=args.impotent, reconnect_on_error=True)

    def sigh(num, frame):
        try:
            if subscription.id:
                subscription.unsubscribe()
            subscription.disconnect()
        except:
            pass

    signal.signal(signal.SIGINT, sigh)
    signal.signal(signal.SIGHUP, sigh)

    if args.daemon:
        if args.logfile:
            logging.basicConfig(filename=args.logfile)
        sys.stdin.close();
        pid = os.fork()
        if pid:
            exit(0)
        os.setsid();
    else:
        logging.basicConfig()

    # Run forever
    subscription.run_callbacks()

if __name__ == "__main__":
    main()
